﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=StaticCode
Version=4.7
@EndOfDesignText@
'Static code module
Sub Process_Globals
	Dim ComparatorMap As Map
	Type ComparatorType(Module As Object,CompareFunction As String)
End Sub
Private Sub GetPackageName As String
	#if B4a
		Return Application.PackageName
	#end if
	#if B4j
		Dim MEJO As JavaObject = Me
		Dim MS As String = MEJO.RunMethod("toString",Null)
		
		Return MS.SubString2(MS.IndexOf(" ")+1,MS.LastIndexOf("."))
	#end if
End Sub

Private Sub ThrowIllegalArgumentException(Msg As String)
	Dim MEJO As JavaObject
	MEJO.InitializeStatic(GetPackageName & ".compareobject")
	MEJO.RunMethod("ThrowIllegalArgumentException",Array(Msg))
End Sub

'Register a type or class comparator 
'Module - The module in which you have created the comparator function
'Example = an example of the type or class
'Compare function = The function to call in the module
Public Sub RegisterComparatorFunction(Module As Object,Example As Object,CompareFunction As String)
	If Not(ComparatorMap.IsInitialized) Then ComparatorMap.Initialize
	Dim CT As ComparatorType
	CT.Initialize
	CT.Module = Module
	CT.CompareFunction = CompareFunction
	ComparatorMap.Put(GetType(Example),CT)
End Sub
'UnRegister a type or class comparator 
'Example = an example of the type or class
Public Sub UnRegisterComparatorFunction(Example As Object)
	If Not(ComparatorMap.IsInitialized) Then Return
	ComparatorMap.Remove(GetType(Example))
End Sub

'Clear all currently stored comparator functions
Public Sub ClearAllComparatorFunctions
	If ComparatorMap.IsInitialized Then ComparatorMap.Clear
End Sub

'Compare Objects.  Will Iterate through Maps,Lists and arrays to find the simple type objects they contain
'You can Register a type or class and provide a comparator to include those in the test.
' If you pass a collection that contains a type or class and don't provide a comparator, an exception will be thrown and logged 
'(just in case you run the test in a try catch)
Public Sub Compare(C1 As Object,C2 As Object) As Boolean
	If GetType(C1) <> GetType(C2) Then
		Dim Msg As String = $"Utils.CompareObjects Error - Type mismatch : ${GetType(C1)} vs ${GetType(C2)}"$
		ThrowIllegalArgumentException(Msg)
		Log("*****")
		Log(Msg)
		Log("*****")
		Return False
	End If
	'Compare Lists
	Dim ThisType As String = GetType(C1)
	
	If ThisType = "java.util.ArrayList"  Or ThisType = "java.util.Arrays$ArrayList" Or ThisType = "anywheresoftware.b4a.objects.collections.List" Then
		If ThisType = "java.util.ArrayList"  Or ThisType = "java.util.Arrays$ArrayList" Then
			Dim L1 As List = C1
			Dim L2 As List = C2
		Else
			Dim L1 As List
			Dim L2 As List
			Dim L3 As JavaObject = C1
			Dim L4 As JavaObject = C2
			L1 = L3.RunMethod("getObject",Null)
			L2 = L4.RunMethod("getObject",Null)
		End If
		If L1.Size <> L2.Size Then Return False
		'If one list is empty, they both must be
		If L1.Size = 0 Then Return True
		For i = 0 To L1.Size - 1
			'Is the item a simple type
			If GetType(L1.Get(i)).StartsWith("java.lang") Then
				If(GetType(L1.Get(i)) <> GetType(L2.Get(i))) Then
					Dim Msg As String = $"CompareObjects Error - Type mismatch in List at index ${i} : ${GetType(L1.Get(i))} Value ${L1.Get(i)} vs ${GetType(L2.Get(i))} Value ${L2.Get(i)}"$
					ThrowIllegalArgumentException(Msg)
					Log("*****")
					Log(Msg)
					Log("*****")
					Return False
				End If
				If L1.Get(i) <> L2.Get(i) Then Return False	
			Else
				If(GetType(L1.Get(i)) <> GetType(L2.Get(i))) Then
					Dim Msg As String = $"CompareObjects Error - Type mismatch in List at index ${i} : ${GetType(L1.Get(i))} Value ${L1.Get(i)} vs ${GetType(L2.Get(i))} Value ${L2.Get(i)}"$
					ThrowIllegalArgumentException(Msg)
					Log("*****")
					Log(Msg)
					Log("*****")
					Return False
				End If
				If Compare(L1.Get(i),L2.Get(i)) = False Then Return False
			End If
		Next
	Else
		'Compare Maps
		If ThisType = "anywheresoftware.b4a.objects.collections.Map$MyMap" Then
			Dim M1 As Map = C1
			Dim M2 As Map = C2
			If M1.Size <> M2.Size Then Return False
			'If one Map is empty, they both must be
			If M1.Size = 0 Then Return True
			For Each Key As Object In M1.Keys
				If GetType(M1.Get(Key)).StartsWith("java.lang") Then
					If M1.Get(Key) <> M2.Get(Key) Then Return False
				Else
					If Compare(M1.Get(Key),M2.Get(Key)) = False Then Return False
				End If
			Next
		Else
			'Compare Arrays they can only contain one type at a time
			If ThisType.StartsWith("[") Then
				Select ThisType.SubString2(1,2)
					Case "I"
						Dim IA1() As Int = C1
						Dim IA2() As Int = C2
						If IA1.Length <> IA2.Length Then Return False
						If IA1.Length = 0 Then Return True
						For i = 0 To IA1.Length - 1
							If IA1(i) <> IA2(i) Then Return False
						Next
					Case "D"
						Dim DA1() As Double = C1
						Dim DA2() As Double = C2
						If DA1.Length <> DA2.Length Then Return False
						If DA1.Length = 0 Then Return True
						For i = 0 To DA1.Length - 1
							If DA1(i) <> DA2(i) Then Return False
						Next
					Case "F"
						Dim FA1() As Float = C1
						Dim FA2() As Float = C2
						If FA1.Length <> FA2.Length Then Return False
						If FA1.Length = 0 Then Return True
						For i = 0 To FA1.Length - 1
							If FA1(i) <> FA2(i) Then Return False
						Next
					Case "C"
						Dim CA1() As Char = C1
						Dim CA2() As Char = C2
						If CA1.Length <> CA2.Length Then Return False
						If CA1.Length = 0 Then Return True
						For i = 0 To CA1.Length - 1
							If CA1(i) <> CA2(i) Then Return False
						Next
					Case "S"
						Dim SA1() As Short = C1
						Dim SA2() As Short = C2
						If SA1.Length <> SA2.Length Then Return False
						If SA1.Length = 0 Then Return True
						For i = 0 To SA1.Length - 1
							If SA1(i) <> SA2(i) Then Return False
						Next
					Case "B"
						Dim BA1() As Byte = C1
						Dim BA2() As Byte = C2
						If BA1.Length <> BA2.Length Then Return False
						If BA1.Length = 0 Then Return True
						For i = 0 To BA1.Length - 1
							If BA1(i) <> BA2(i) Then Return False
						Next
					Case "Z"
						Dim ZA1() As Boolean = C1
						Dim ZA2() As Boolean = C2
						If ZA1.Length <> ZA2.Length Then Return False
						If ZA1.Length = 0 Then Return True
						For i = 0 To ZA1.Length - 1
							If ZA1(i) <> ZA2(i) Then Return False
						Next
					Case Else
						Dim A1() As Object = C1
						Dim A2() As Object = C2
						If A1.Length <> A2.Length Then Return False
						If A1.Length = 0 Then Return True
						'If the array contains simple types
						If GetType(A1(0)).StartsWith("java.lang") Then
							For i = 0 To A1.Length - 1
								If A1(i) <> A2(i) Then Return False
							Next
						Else
							For i = 0 To A1.Length - 1
								If Compare(A1(i),A2(i)) = False Then Return False
							Next
						End If
				
				End Select
			Else
				'Check everything else is a simple type object or Class or UserDefined Type
				If ThisType.StartsWith("java.lang") = False Then
					'Types and Classes, Need to have registered a comaprator for each type of these that the collection contains.
					If ComparatorMap.IsInitialized Then
						Dim CT As ComparatorType = ComparatorMap.Get(GetType(C1))
						If CT <> Null Then
							If SubExists(CT.Module, CT.CompareFunction) Then
								Return CallSub3(CT.Module,CT.CompareFunction,C1,C2)
							Else
								Dim Msg As String = $"CompareObjects Error - Type or class ${ThisType} found, but comparator function ${CT.CompareFunction} not found in specified module"$
								ThrowIllegalArgumentException(Msg)
								Log("*****")
								Log(Msg)
								Log("*****")
								Return False
							End If
						End If
					Else
						Dim Msg As String = $"CompareObjects Error - Unknown Type or class ${ThisType} found. If it's a Class or Type did you forget to register a comparator?"$
						ThrowIllegalArgumentException(Msg)
						Log("*****")
						Log(Msg)
						Log("*****")
						Return False
					End If
				Else
					'Compare java.lang types else
					If(GetType(C1) <> GetType(C2)) Then
						Dim Msg As String = $"CompareObjects Error - Type mismatch  : ${GetType(C1)} Value ${C1} vs ${GetType(C2)} Value ${C2}"$
						ThrowIllegalArgumentException(Msg)
						Log("*****")
						Log(Msg)
						Log("*****")
						Return False
					End If
					If(C1 <> C2) Then Return False
				End If
			End If
		End If
		
	End If
	Return True
End Sub

#if java

public static Object[] convertObject(Object Obj){
	return (Object[]) Obj;
}

public static void ThrowIllegalArgumentException(String msg){
	throw new IllegalArgumentException(msg);
}
#end if